import { __decorate, __awaiter } from 'tslib';
import { ɵɵdefineInjectable, Injectable, EventEmitter, Input, Output, Component, HostListener, NgModule } from '@angular/core';
import { Subject } from 'rxjs';
import { CommonModule } from '@angular/common';

let LandmarksService = class LandmarksService {
    constructor() {
        this.$landmarks = new Subject();
    }
    setLandmarks(landmarks) {
        this.$landmarks.next(landmarks);
    }
    get landmarks() {
        return this.$landmarks.asObservable();
    }
};
LandmarksService.ngInjectableDef = ɵɵdefineInjectable({ factory: function LandmarksService_Factory() { return new LandmarksService(); }, token: LandmarksService, providedIn: "root" });
LandmarksService = __decorate([
    Injectable({
        providedIn: 'root'
    })
], LandmarksService);

class ResponseAction {
    constructor(event) {
        this.event = event;
    }
}

class ResponseError {
    constructor(error, code) {
        this.error = error;
        this.code = code;
    }
}

var EventAction;
(function (EventAction) {
    EventAction["LEFT"] = "LEFT";
    EventAction["RIGHT"] = "RIGHT";
    EventAction["FRONT"] = "FRONT";
    EventAction["SMILE"] = "SMILE";
})(EventAction || (EventAction = {}));

class EventComplete {
    constructor(gesture) {
        this.gesture = gesture;
    }
}

var ErrorType;
(function (ErrorType) {
    ErrorType["WEBGL_EXT_NOT_SUPPORTED"] = "Este navegador no soporta prueba de vida: extensi\u00F3n webgl no soportada";
    ErrorType["GESTURE_NOT_FOUND"] = "Error al validar los gestos seleccionados, gesto no reconocido: ";
    ErrorType["BROWSER_NOT_SUPPORTED"] = "Navegador no soportado";
    ErrorType["NOT_ACCEPT_CAMERA_PERMISSION"] = "No tienes permisos para acceder a la c\u00E1mara";
    ErrorType["MAX_TIME_LIMIT"] = "Tiempo m\u00E1ximo superado";
    ErrorType["WORKER_PATH_REQUIRED"] = "workerPath es requerido";
    ErrorType["WORKER_DEPENDENCIES_ERRROR"] = "Dependencia faltante: ";
})(ErrorType || (ErrorType = {}));

var ErrorCode;
(function (ErrorCode) {
    ErrorCode[ErrorCode["WEBGL_EXT_NOT_SUPPORTED"] = -1] = "WEBGL_EXT_NOT_SUPPORTED";
    ErrorCode[ErrorCode["GESTURE_NOT_FOUND"] = -2] = "GESTURE_NOT_FOUND";
    ErrorCode[ErrorCode["BROWSER_NOT_SUPPORTED"] = -3] = "BROWSER_NOT_SUPPORTED";
    ErrorCode[ErrorCode["NOT_ACCEPT_CAMERA_PERMISSION"] = -4] = "NOT_ACCEPT_CAMERA_PERMISSION";
    ErrorCode[ErrorCode["MAX_TIME_LIMIT"] = -5] = "MAX_TIME_LIMIT";
    ErrorCode[ErrorCode["WORKER_PATH_REQUIRED"] = -6] = "WORKER_PATH_REQUIRED";
    ErrorCode[ErrorCode["WORKER_DEPENDENCIES_ERRROR"] = -7] = "WORKER_DEPENDENCIES_ERRROR";
})(ErrorCode || (ErrorCode = {}));

class Point {
    constructor(x, y) {
        this.x = x;
        this.y = y;
    }
}

class Landmarks {
    constructor(eyeRight, eyeLeft, nose, mouth, jawRight, jawLeft) {
        this.eyeRight = new Point(eyeRight[0], eyeRight[1]);
        this.eyeLeft = new Point(eyeLeft[0], eyeLeft[1]);
        this.nose = new Point(nose[0], nose[1]);
        this.mouth = new Point(mouth[0], mouth[1]);
        this.jawRight = new Point(jawRight[0], jawRight[1]);
        this.jawLeft = new Point(jawLeft[0], jawLeft[1]);
    }
}

class SampleResult {
    constructor() {
    }
}

var Mode;
(function (Mode) {
    Mode["NEUTRAL"] = "NEUTRAL";
    Mode["LOW"] = "LOW";
    Mode["MEDIUM"] = "MEDIUM";
    Mode["HIGH"] = "HIGH";
})(Mode || (Mode = {}));

var Side;
(function (Side) {
    Side["LEFT"] = "LEFT";
    Side["RIGHT"] = "RIGHT";
})(Side || (Side = {}));

const POSITION_NEUTRAL = 50;
const THRESHOLD_NEUTRAL_WIDTH = 8;
const THRESHOLD_NEUTRAL_HEIGHT = 8;
const MAX_STD_DEVIATION = 3;
const THRESHOLD_MODE = [
    { name: [Mode.NEUTRAL], value: 30 }, { name: [Mode.LOW], value: 45 },
    { name: [Mode.MEDIUM, Mode.LOW], value: 85 }, { name: [Mode.HIGH, Mode.MEDIUM, Mode.LOW], value: 100 }
];
class Utils {
    constructor() { }
    static validateSample(data) {
        const landmarks = this.getLandmarks(data);
        const validateNeutralMouthNose = () => {
            const MIN_THRESHOLD = POSITION_NEUTRAL - THRESHOLD_NEUTRAL_WIDTH;
            const MAX_THRESHOLD = POSITION_NEUTRAL + THRESHOLD_NEUTRAL_WIDTH;
            const boundingBox = Utils.getBoundingBox(data.topLeft, data.bottomRight);
            const proportionMouth = Utils.getProportion(boundingBox, landmarks.mouth);
            const proportionNose = Utils.getProportion(boundingBox, landmarks.nose);
            if (proportionMouth.width >= MIN_THRESHOLD && proportionMouth.width <= MAX_THRESHOLD
                && proportionNose.width >= MIN_THRESHOLD && proportionNose.width <= MAX_THRESHOLD)
                return true;
            else {
                console.log('mouth and nose', proportionMouth, proportionNose);
                return false;
            }
        };
        return this.validateNeutralPosition(landmarks) && validateNeutralMouthNose();
    }
    static analyzeSamples(sample) {
        const dataLandmarks = [];
        dataLandmarks[0] = new Array(); // eyeRight
        dataLandmarks[1] = new Array(); // eyeLeft
        dataLandmarks[2] = new Array(); // nose
        dataLandmarks[3] = new Array(); // mouth
        dataLandmarks[4] = new Array(); // jawRight
        dataLandmarks[5] = new Array(); // jawLeft
        const getMean = (data) => {
            let sumWidth = 0;
            let sumHeight = 0;
            data.forEach(element => {
                sumWidth += element.width;
                sumHeight += element.height;
            });
            return { width: sumWidth / data.length, height: sumHeight / data.length };
        };
        const getStdDeviation = (data, mean) => {
            let sumWidth = 0;
            let sumHeight = 0;
            data.forEach(element => {
                sumWidth += Math.pow(element.width - mean.width, 2);
                sumHeight += Math.pow(element.height - mean.height, 2);
            });
            const stdDeviationWidth = Math.sqrt(sumWidth / (data.length));
            const stdDeviationHeight = Math.sqrt(sumHeight / (data.length));
            return { width: stdDeviationWidth, height: stdDeviationHeight };
        };
        const isValidStdDeviation = (stdDeviation) => {
            if (stdDeviation.width <= MAX_STD_DEVIATION || stdDeviation.height <= MAX_STD_DEVIATION)
                return true;
            else {
                console.log('standar deviation: ' + stdDeviation.width + ' - ' + stdDeviation.height);
                return false;
            }
        };
        const getSampleResult = () => {
            let result = new SampleResult();
            dataLandmarks.every((element, index) => {
                const mean = getMean(element);
                const delta = getStdDeviation(element, mean);
                if (isValidStdDeviation(delta)) {
                    const temp = { width: mean.width, height: mean.height, deltaWidth: delta.width, deltaHeight: delta.height };
                    switch (index) {
                        case 0:
                            result.eyeRight = temp;
                            break;
                        case 1:
                            result.eyeLeft = temp;
                            break;
                        case 2:
                            result.nose = temp;
                            break;
                        case 3:
                            result.mouth = temp;
                            break;
                        case 4:
                            result.jawRight = temp;
                            break;
                        case 5:
                            result.jawLeft = temp;
                            break;
                    }
                    return true;
                }
                else {
                    result = null;
                    return false;
                }
            });
            console.log(result);
            return result;
        };
        sample.forEach((element) => {
            const boundingBox = Utils.getBoundingBox(element.topLeft, element.bottomRight);
            const landmarks = new Landmarks(element.landmarks[0], element.landmarks[1], element.landmarks[2], // eyeRight, eyeLeft, nose
            element.landmarks[3], element.landmarks[4], element.landmarks[5] // mouth, jawRight, jawLeft
            );
            dataLandmarks[0].push(Utils.getProportion(boundingBox, landmarks.eyeRight));
            dataLandmarks[1].push(Utils.getProportion(boundingBox, landmarks.eyeLeft));
            dataLandmarks[2].push(Utils.getProportion(boundingBox, landmarks.nose));
            dataLandmarks[3].push(Utils.getProportion(boundingBox, landmarks.mouth));
            dataLandmarks[4].push(Utils.getProportion(boundingBox, landmarks.jawRight));
            dataLandmarks[5].push(Utils.getProportion(boundingBox, landmarks.jawLeft));
        });
        return getSampleResult();
    }
    static validateNeutralFace(data, sample) {
        const landmarks = this.getLandmarks(data);
        const validateLandmark = (landmark, sampleLandmark) => {
            const value = Utils.getProportion(boundingBox, landmark);
            const minWidth = sampleLandmark.width - (sampleLandmark.deltaWidth + THRESHOLD_NEUTRAL_WIDTH);
            const maxWidth = sampleLandmark.width + (sampleLandmark.deltaWidth + THRESHOLD_NEUTRAL_WIDTH);
            const minHeight = sampleLandmark.height - (sampleLandmark.deltaHeight + THRESHOLD_NEUTRAL_HEIGHT);
            const maxHeight = sampleLandmark.height + (sampleLandmark.deltaHeight + THRESHOLD_NEUTRAL_HEIGHT);
            if (value.width >= minWidth && value.width <= maxWidth
                && value.height >= minHeight && value.height <= maxHeight)
                return true;
            else
                return false;
        };
        const boundingBox = Utils.getBoundingBox(data.topLeft, data.bottomRight);
        if (this.validateNeutralPosition(landmarks) && validateLandmark(landmarks.mouth, sample.mouth) && validateLandmark(landmarks.nose, sample.nose))
            return true;
        else
            return false;
    }
    static validateTurn(data, sample, mode, side) {
        const landmarks = this.getLandmarks(data);
        const getProportionNose = () => {
            const boundingBox = Utils.getBoundingBox(data.topLeft, data.bottomRight);
            return Utils.getProportion(boundingBox, landmarks.nose);
        };
        const getSide = () => {
            if (nose.width > sample.nose.width)
                return Side.LEFT;
            else
                return Side.RIGHT;
        };
        const getEquivalence = () => {
            if (side === Side.LEFT) {
                const total = 100 - sample.nose.width;
                const noseProportional = nose.width - sample.nose.width;
                return noseProportional * 100 / total;
            }
            else {
                const noseProportional = nose.width * 100 / sample.nose.width;
                return 100 - noseProportional;
            }
        };
        const validateMode = () => {
            const equivalence = getEquivalence();
            const currentMode = THRESHOLD_MODE.find(element => equivalence < element.value);
            if (currentMode && currentMode.name.includes(mode)) {
                console.log('--------------------------------- ');
                console.log(currentMode.name);
                console.log('--------------------------------- ');
                return true;
            }
            else {
                if (currentMode && currentMode.name && currentMode.name[0] !== Mode.NEUTRAL) {
                    console.log('mode incorrect: ' + currentMode.name[0]);
                }
                return false;
            }
        };
        const nose = getProportionNose();
        const boundingBox = Utils.getBoundingBox(data.topLeft, data.bottomRight);
        if (side !== getSide()) {
            console.log('side incorrect: ' + getSide());
        }
        if (side === getSide() && validateMode() && this.validateFaceProportionHeigth(landmarks, boundingBox, sample))
            return true;
        else
            return false;
    }
    static validateFaceProportionHeigth(landmarks, boundingBox, sample) {
        const validateLandmark = (landmark, sampleLandmark, factor = 2) => {
            const value = Utils.getProportion(boundingBox, landmark);
            const minHeight = sampleLandmark.height - (sampleLandmark.deltaHeight + THRESHOLD_NEUTRAL_HEIGHT + factor);
            const maxHeight = sampleLandmark.height + (sampleLandmark.deltaHeight + THRESHOLD_NEUTRAL_HEIGHT + factor);
            if (value.height >= minHeight && value.height <= maxHeight)
                return true;
            else {
                if (value.height <= minHeight) {
                    console.log('incorrect min  h: ' + value.height + ' - min h:' + minHeight + ' error: ' + (minHeight - value.height));
                }
                if (value.height >= maxHeight) {
                    console.log('incorrect max h: ' + value.height + ' - max h: ' + maxHeight + ' error: ' + (value.height - maxHeight));
                }
                return false;
            }
        };
        const validMouth = validateLandmark(landmarks.mouth, sample.mouth, 2);
        const validNose = validateLandmark(landmarks.nose, sample.nose, 3);
        const validEyeLeft = validateLandmark(landmarks.eyeLeft, sample.eyeLeft, 2);
        const validEyeRight = validateLandmark(landmarks.eyeRight, sample.eyeRight, 2);
        const validJawLeft = validateLandmark(landmarks.jawLeft, sample.jawLeft, 2);
        const validJawRight = validateLandmark(landmarks.jawRight, sample.jawRight, 2);
        if (validMouth && validNose && validEyeLeft && validEyeRight && validJawLeft && validJawRight)
            return true;
        else {
            console.log('validMouth: ' + validMouth + '- validNose: ' + validNose + '- validEyeLeft: ' + validEyeLeft
                + '- validEyeRight:  ' + validEyeRight + '-  validJawLeft: ' + validJawLeft + ' - validJawRight: ' + validJawRight);
            return false;
        }
    }
    static getLandmarks(data) {
        data.landmarks.forEach(landmark => {
            landmark[0] = landmark[0] - data.topLeft[0];
            landmark[1] = landmark[1] - data.topLeft[1];
        });
        const landmarks = new Landmarks(data.landmarks[0], data.landmarks[1], data.landmarks[2], // eyeRight, eyeLeft, nose
        data.landmarks[3], data.landmarks[4], data.landmarks[5] // mouth, jawRight, jawLeft
        );
        return landmarks;
    }
    static getBoundingBox(topLeft, bottomRight) {
        return { width: bottomRight[0] - topLeft[0], height: bottomRight[1] - topLeft[1] };
    }
    static validateNeutralPosition(landmarks) {
        const validateNeutralEyes = () => {
            if (landmarks.jawLeft.x > landmarks.eyeLeft.x && landmarks.eyeLeft.x > landmarks.eyeRight.x && landmarks.eyeRight.x > landmarks.jawRight.x)
                return true;
            else
                return false;
        };
        const validateNeutralMouth = () => {
            if (landmarks.mouth.x < landmarks.eyeLeft.x && landmarks.mouth.x > landmarks.eyeRight.x)
                return true;
            else
                return false;
        };
        const validateNeutralNose = () => {
            if (landmarks.nose.x < landmarks.eyeLeft.x && landmarks.nose.x > landmarks.eyeRight.x)
                return true;
            else
                return false;
        };
        const validateNeutralMouthNose = () => {
            if (landmarks.mouth.y > landmarks.nose.y)
                return true;
            else
                return false;
        };
        if (validateNeutralEyes() && validateNeutralMouth() && validateNeutralNose() && validateNeutralMouthNose())
            return true;
        else {
            // console.log('incorrec neutral posotion:' +  ' eyes: ' + validateNeutralEyes() + ', mouth: ' +  validateNeutralMouth() + ', nose: ' + validateNeutralNose() + ', mouth and nose: ' + validateNeutralMouthNose());
            return false;
        }
    }
    static getProportion(boundingBox, point) {
        const proportionWidth = (point.x * 100) / boundingBox.width;
        const proportionHeight = (point.y * 100) / boundingBox.height;
        return { width: proportionWidth, height: proportionHeight };
    }
    static getImage() {
        const image = new Image();
        image.src = './assets/test.png';
        return image;
    }
}

const GESTURES = ['LEFT', 'RIGHT', 'SMILE'];
const DEPENDENCIES_FILES = ['liveness.worker.js', 'tf-core.min.js', 'tf-backend-wasm.min.js',
    'tfjs-backend-wasm.wasm', 'tf-converter.min.js', 'blazeface.min.js'];
const PROGRESS_CONFIGURATION = { radius: 29, color: { start: '#F4931D', end: '#5A9A92' }, center: 45 };
let NgNaatLivenessComponent = class NgNaatLivenessComponent {
    // image = Utils.getImage();
    constructor(landmarksService) {
        this.landmarksService = landmarksService;
        this.videoMirror = true;
        this.sampleSize = 25;
        this.mode = Mode.MEDIUM;
        this.mask = true;
        this.onerror = new EventEmitter();
        this.detectionReady = new EventEmitter();
        this.calibrationPercentage = new EventEmitter();
        this.calibrationReady = new EventEmitter();
        this.faceDetected = new EventEmitter();
        this.actionComplete = new EventEmitter();
        this.gestureComplete = new EventEmitter();
        this.processComplete = new EventEmitter();
        this.events = [];
        // face detected
        this.isFaceDetected = false;
        // process complete
        this.isProcessComplete = false;
        // samples face
        this.sampleData = [];
        this.workerReady = new Subject();
        this.workerDtetectedFace = new Subject();
        // progress
        this.progressPercentage = 0;
        this.progressComplete = false;
        this.getSampleData = () => __awaiter(this, void 0, void 0, function* () {
            const faces = yield this.estimateFaces();
            if (faces.length > 0 && Utils.validateSample(faces[0])) {
                this.sampleData.push(faces[0]);
                this.progressPercentage = Math.round(this.sampleData.length * 100 / this.sampleSize);
                this.calibrationPercentage.next(this.progressPercentage);
            }
            if (this.sampleData.length === this.sampleSize) { // validate size of samples
                this.sampleResult = Utils.analyzeSamples(this.sampleData); // analyze samples
                if (this.sampleResult) { // validate integrity of the samples
                    this.progressComplete = true;
                    this.calibrationReady.next();
                    this.initEvents();
                }
                else {
                    console.log('get sample again...');
                    this.sampleData = [];
                    this.getSampleData();
                }
            }
            else {
                requestAnimationFrame(this.getSampleData);
            }
        });
        this.detectFace = () => __awaiter(this, void 0, void 0, function* () {
            const faces = yield this.estimateFaces();
            if (faces.length > 0) {
                this.faceDetected.next(true);
                this.isFaceDetected = true;
                if (this.drawFace)
                    this.draw(faces[0]);
                if (this.activeEmiterEvent)
                    this.landmarksService.setLandmarks(faces[0]);
            }
            else {
                this.isFaceDetected = false;
                this.faceDetected.next(false);
            }
            if (!this.isProcessComplete)
                requestAnimationFrame(this.detectFace);
        });
        console.log('WORKERS VERSION 7.0');
    }
    ngOnInit() {
        this.initElements();
        Promise.all([this.validateDependenciesWorker(), this.validateGestures(), this.getUserMedia(), this.loadWorker()]).then(values => {
            this.stream = values[2]; // getUserMedia
            this.video.srcObject = this.stream;
            this.gestureComplete.subscribe((res) => {
                this.validateActionComplete(res.event);
            });
        }).catch(error => this.onerror.emit(error));
    }
    initElements() {
        this.video = document.getElementById('video');
        this.canvas = document.getElementById('overlay');
        this.canvasWorker = document.createElement('canvas');
        this.canvasContext = this.canvas.getContext('2d');
        this.canvasContextWorker = this.canvasWorker.getContext('2d');
        const container = document.getElementById('container-liveness');
        if (this.mask)
            this.initMask();
        if (this.videoAlign)
            container.style.alignItems = this.videoAlign;
        this.video.onloadeddata = () => this.videoLoaded();
    }
    initMask() {
        this.progressConfiguration = Object.assign(PROGRESS_CONFIGURATION, this.progressConfiguration);
    }
    validateDependenciesWorker() {
        return new Promise((resolve, reject) => {
            const files = DEPENDENCIES_FILES.map(element => `${this.workerPath}${element}`);
            if (this.workerPath) {
                Promise.all([fetch(files[0]), fetch(files[1]), fetch(files[2]), fetch(files[3]), fetch(files[4]), fetch(files[5]),]).then(response => {
                    const notFound = response.find(element => element.status === 404);
                    if (notFound)
                        reject(new ResponseError(ErrorType.WORKER_DEPENDENCIES_ERRROR + notFound.url, ErrorCode.WORKER_DEPENDENCIES_ERRROR));
                    else
                        resolve();
                }).catch(() => reject(new ResponseError(ErrorType.WORKER_DEPENDENCIES_ERRROR, ErrorCode.WORKER_DEPENDENCIES_ERRROR)));
            }
            else
                reject(new ResponseError(ErrorType.WORKER_PATH_REQUIRED, ErrorCode.WORKER_PATH_REQUIRED));
        });
    }
    validateGestures() {
        return new Promise((resolve, reject) => {
            this.gestures.forEach(element => {
                this.events.push(new EventComplete(element.toUpperCase()));
                if (!GESTURES.includes(element.toUpperCase()))
                    reject(new ResponseError(ErrorType.GESTURE_NOT_FOUND + element, ErrorCode.GESTURE_NOT_FOUND));
            });
            resolve();
        });
    }
    getUserMedia() {
        return new Promise((resolve, reject) => {
            if (navigator.mediaDevices && navigator.mediaDevices.getUserMedia) {
                navigator.mediaDevices.getUserMedia(this.videoConstraints ? this.videoConstraints : { video: true }).then((stream) => resolve(stream))
                    .catch(() => reject(new ResponseError(ErrorType.NOT_ACCEPT_CAMERA_PERMISSION, ErrorCode.NOT_ACCEPT_CAMERA_PERMISSION)));
            }
            else
                reject(new ResponseError(ErrorType.BROWSER_NOT_SUPPORTED, ErrorCode.BROWSER_NOT_SUPPORTED));
        });
    }
    loadWorker() {
        return new Promise((resolve) => {
            this.initWorker();
            this.workerReady.subscribe(() => resolve());
        });
    }
    initWorker() {
        this.worker = new Worker(this.workerPath + DEPENDENCIES_FILES[0]);
        this.worker.onmessage = ((response) => {
            if (response.data.event === 'loadModel')
                this.workerReady.next();
            if (response.data.event === 'detectFace')
                this.workerDtetectedFace.next(response.data.faces);
        });
        this.worker.postMessage({ event: 'loadModel' });
    }
    videoLoaded() {
        this.canvas.width = this.video.videoWidth;
        this.canvas.height = this.video.videoHeight;
        this.canvasWorker.width = this.video.videoWidth / 2;
        this.canvasWorker.height = this.video.videoHeight / 2;
        this.firstDetection();
    }
    firstDetection() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.estimateFaces();
            this.detectionReady.next();
            console.log('get sample...');
            this.getSampleData();
        });
    }
    estimateFaces() {
        return new Promise((resolve) => {
            this.canvasContextWorker.clearRect(0, 0, this.canvasWorker.width, this.canvasWorker.height);
            this.canvasContextWorker.drawImage(this.video, 0, 0, this.canvasWorker.width, this.canvasWorker.height);
            const data = { event: 'detectFace', data: this.canvasContextWorker.getImageData(0, 0, this.canvasWorker.width, this.canvasWorker.height) };
            const subcription = this.workerDtetectedFace.subscribe(faces => {
                subcription.unsubscribe();
                resolve(faces);
            });
            this.worker.postMessage(data);
        });
    }
    initEvents() {
        console.log('init events');
        this.detectFace();
        this.listenEvents();
        if (this.timer)
            this.startTimer();
    }
    draw(predictions) {
        this.canvasContext.clearRect(0, 0, this.canvas.width, this.canvas.height);
        const start = predictions.topLeft;
        const end = predictions.bottomRight;
        const size = [end[0] - start[0], end[1] - start[1]];
        this.canvasContext.fillStyle = 'rgba(255, 255, 255, 0.5)';
        this.canvasContext.fillRect(start[0], start[1], size[0], size[1]);
        const landmarks = predictions.landmarks;
        this.canvasContext.fillStyle = 'blue';
        let count = 0;
        for (const landmark of landmarks) {
            const x = landmark[0];
            const y = landmark[1];
            this.canvasContext.fillRect(x, y, 5, 5);
            // this.canvasContext.fillText(`${count}`, x, y);
            count++;
        }
    }
    listenEvents() {
        if (this.events.filter(element => element.complete).length === this.gestures.length) {
            this.video.pause();
            this.getCapture().then(res => {
                this.processComplete.next(res);
                this.ngOnDestroy();
            });
        }
        else {
            const currentEvent = this.events.find(element => !element.complete);
            this.executeEvent(currentEvent.gesture);
        }
    }
    validateActionComplete(event) {
        const currentEvent = this.events.find(element => !element.complete);
        if (event === currentEvent.gesture) {
            currentEvent.complete = true;
            this.listenEvents();
        }
    }
    executeEvent(gesture) {
        if (gesture === 'LEFT')
            this.listenTurnLeft();
        else if (gesture === 'RIGHT')
            this.listenTurnRight();
        // else if (gesture === 'SMILE') this.listenSmile();
    }
    listenTurnLeft() {
        this.activeEmiterEvent = true;
        this.pointsSubscriptionLeft = this.landmarksService.landmarks.subscribe(response => {
            if (this.isFaceDetected && Utils.validateTurn(response, this.sampleResult, this.mode, Side.LEFT)) {
                this.actionComplete.next(new ResponseAction(EventAction.LEFT));
                this.pointsSubscriptionLeft.unsubscribe();
                this.listenFront(true);
            }
        });
    }
    listenTurnRight() {
        this.activeEmiterEvent = true;
        this.pointsSubscriptionRight = this.landmarksService.landmarks.subscribe(response => {
            if (this.isFaceDetected && Utils.validateTurn(response, this.sampleResult, this.mode, Side.RIGHT)) {
                this.actionComplete.next(new ResponseAction(EventAction.RIGHT));
                this.pointsSubscriptionRight.unsubscribe();
                this.listenFront(false);
            }
        });
    }
    listenFront(isLeft) {
        this.pointsSubscriptionFront = this.landmarksService.landmarks.subscribe(response => {
            if (this.isFaceDetected && Utils.validateNeutralFace(response, this.sampleResult)) {
                this.activeEmiterEvent = false;
                this.pointsSubscriptionFront.unsubscribe();
                this.actionComplete.next(new ResponseAction(EventAction.FRONT));
                this.gestureComplete.next(new ResponseAction(isLeft ? EventAction.LEFT : EventAction.RIGHT));
            }
        });
    }
    listenSmile() {
    }
    getCapture() {
        // TODO: validate image before send
        return new Promise((resolve, reject) => {
            try {
                const canvas = document.createElement('canvas');
                const ctx = canvas.getContext('2d');
                canvas.width = this.video.videoWidth;
                canvas.height = this.video.videoHeight;
                ctx.drawImage(this.video, 0, 0);
                resolve(canvas.toDataURL());
            }
            catch (error) {
                reject(error);
            }
        });
    }
    startTimer() {
        this.timerTimeout = setTimeout(() => {
            this.ngOnDestroy();
            this.onerror.next(new ResponseError(ErrorType.MAX_TIME_LIMIT, ErrorCode.MAX_TIME_LIMIT));
        }, this.timer * 1000);
    }
    cameraOff() {
        if (this.stream) {
            this.stream.getTracks().forEach(track => {
                track.stop();
            });
        }
    }
    ngOnDestroy() {
        this.cameraOff();
        if (this.timerTimeout)
            clearTimeout(this.timerTimeout);
        this.isProcessComplete = true;
        if (this.pointsSubscriptionLeft)
            this.pointsSubscriptionLeft.unsubscribe();
        if (this.pointsSubscriptionRight)
            this.pointsSubscriptionRight.unsubscribe();
        if (this.pointsSubscriptionFront)
            this.pointsSubscriptionFront.unsubscribe();
    }
};
NgNaatLivenessComponent.ctorParameters = () => [
    { type: LandmarksService }
];
__decorate([
    Input()
], NgNaatLivenessComponent.prototype, "drawFace", void 0);
__decorate([
    Input()
], NgNaatLivenessComponent.prototype, "videoMirror", void 0);
__decorate([
    Input()
], NgNaatLivenessComponent.prototype, "gestures", void 0);
__decorate([
    Input()
], NgNaatLivenessComponent.prototype, "timer", void 0);
__decorate([
    Input()
], NgNaatLivenessComponent.prototype, "sampleSize", void 0);
__decorate([
    Input()
], NgNaatLivenessComponent.prototype, "workerPath", void 0);
__decorate([
    Input()
], NgNaatLivenessComponent.prototype, "mode", void 0);
__decorate([
    Input()
], NgNaatLivenessComponent.prototype, "modelPath", void 0);
__decorate([
    Input()
], NgNaatLivenessComponent.prototype, "mask", void 0);
__decorate([
    Input()
], NgNaatLivenessComponent.prototype, "progressConfiguration", void 0);
__decorate([
    Input()
], NgNaatLivenessComponent.prototype, "videoAlign", void 0);
__decorate([
    Input()
], NgNaatLivenessComponent.prototype, "videoConstraints", void 0);
__decorate([
    Output()
], NgNaatLivenessComponent.prototype, "onerror", void 0);
__decorate([
    Output()
], NgNaatLivenessComponent.prototype, "detectionReady", void 0);
__decorate([
    Output()
], NgNaatLivenessComponent.prototype, "calibrationPercentage", void 0);
__decorate([
    Output()
], NgNaatLivenessComponent.prototype, "calibrationReady", void 0);
__decorate([
    Output()
], NgNaatLivenessComponent.prototype, "faceDetected", void 0);
__decorate([
    Output()
], NgNaatLivenessComponent.prototype, "actionComplete", void 0);
__decorate([
    Output()
], NgNaatLivenessComponent.prototype, "gestureComplete", void 0);
__decorate([
    Output()
], NgNaatLivenessComponent.prototype, "processComplete", void 0);
NgNaatLivenessComponent = __decorate([
    Component({
        selector: 'ng-naat-liveness',
        template: "<ng-content select=\"[body]\"></ng-content>\n<div id=\"container-liveness\" [ngClass]=\"{'videoRotate': videoMirror}\">\n  <video id=\"video\" autoplay muted playsinline></video>\n  <canvas id=\"overlay\"></canvas>\n</div>\n<ng-naat-liveness-progress *ngIf=\"mask\" [value]=\"progressPercentage\" [progressComplete]=\"progressComplete\"\n  [configuration]=\"progressConfiguration\"></ng-naat-liveness-progress>",
        styles: [":host{width:100%;height:100%}#container-liveness{display:flex;justify-content:center;align-items:center;width:100%;height:100%;overflow:hidden}#video{max-width:100%;max-height:100%}#overlay{max-width:100%;position:fixed}.videoRotate{transform:rotateY(180deg)}"]
    })
], NgNaatLivenessComponent);

let ProgressComponent = class ProgressComponent {
    constructor() {
        this.circumference = 0;
        this.radiusReady = new Subject();
        this.rotationReady = true;
        this.pathMask = `url(${window.location.href}#mask-opacity)`;
    }
    onorientationchange(event) {
        this.renderSvgContainer();
    }
    ngOnInit() {
        this.setColor();
    }
    ngAfterViewInit() {
        this.initElements();
        this.radiusReady.subscribe((radius) => {
            this.circumference = 2 * Math.PI * Number(radius);
            this.progress(0);
        });
    }
    renderSvgContainer() {
        const svgProgress = document.getElementById('svg-progress');
        clearTimeout(this.timeoutOrientation);
        svgProgress.style.display = 'none';
        this.rotationReady = false;
        this.timeoutOrientation = setTimeout(() => {
            if (svgProgress.style.display === 'block')
                svgProgress.style.display = 'none';
            svgProgress.style.display = 'block';
            this.rotationReady = true;
        }, 2000);
    }
    initElements() {
        this.circle = document.getElementById('progress-bar');
        this.getRadius();
    }
    getRadius() {
        const radius = this.circle && this.circle.r ? this.circle.r.baseVal.value : 0;
        if (radius !== 0)
            this.radiusReady.next(radius);
        else
            setTimeout(() => this.getRadius(), 100);
    }
    ngOnChanges(changes) {
        if (changes.value && changes.value.currentValue !== changes.value.previousValue && changes.value.currentValue !== 0) {
            this.progress(changes.value.currentValue);
        }
    }
    setColor() {
        if (typeof this.configuration.color === 'string') {
            this.color = { start: this.configuration.color, end: this.configuration.color };
        }
        else
            this.color = { start: this.configuration.color.start, end: this.configuration.color.end };
    }
    progress(value) {
        if (value < 60 && this.circle.getAttribute('stroke') !== this.color.start)
            this.circle.setAttribute('stroke', this.color.start);
        else if (value >= 60 && this.circle.getAttribute('stroke') !== this.color.end)
            this.circle.setAttribute('stroke', this.color.end);
        const progress = value / 100;
        this.dashoffset = this.circumference * (1 - progress);
    }
};
__decorate([
    Input()
], ProgressComponent.prototype, "value", void 0);
__decorate([
    Input()
], ProgressComponent.prototype, "progressComplete", void 0);
__decorate([
    Input()
], ProgressComponent.prototype, "configuration", void 0);
__decorate([
    HostListener('window:orientationchange', ['$event'])
], ProgressComponent.prototype, "onorientationchange", null);
ProgressComponent = __decorate([
    Component({
        selector: 'ng-naat-liveness-progress',
        template: "\n    <svg class=\"svg-progress\" id=\"svg-progress\">\n        <defs>\n            <mask id=\"mask-opacity\" height=\"100%\" width=\"100%\" x=\"0\" y=\"0\">\n                <rect height=\"100%\" width=\"100%\" x=\"0\" y=\"0\" fill=\"#bbbbbb\"></rect>\n                <circle cx=\"50%\" attr.cy=\"{{configuration.center}}%\"  attr.r=\"{{configuration.radius}}%\" />\n            </mask>\n        </defs>\n        <rect stroke-opacity=\"1\" height=\"100%\" width=\"100%\" attr.mask=\"{{pathMask}}\"  width=\"100%\" x=\"0\" y=\"0\"></rect>\n    </svg>\n    \n    \n    <svg class=\"svg-progress\" [ngClass]=\"{'progress-complete': progressComplete}\" style=\"z-index: 1\">\n        <circle id=\"progress-bar\" [style.strokeDasharray]=\"circumference\" attr.stroke=\"{{color.start}}\"\n            [style.strokeDashoffset]=\"dashoffset\" cx=\"50%\" attr.cy=\"{{configuration.center}}%\"  attr.r=\"{{configuration.radius + 2}}%\" stroke-width=\"5\" fill=\"none\">\n        </circle>\n    </svg>\n    \n    \n    <svg class=\"svg-progress\" [ngClass]=\"{'progress-complete': progressComplete}\">\n        <circle cx=\"50%\" attr.cy=\"{{configuration.center}}%\"  stroke-width=\"5\" attr.r=\"{{configuration.radius + 2}}%\" stroke=\"#afafaf\" fill=\"none\" />\n    </svg>\n\n    <!-- init black screen -->\n    <div *ngIf=\"!rotationReady\" class=\"waiting\">\n            <span> Iniciando... </span>\n    </div>",
        styles: [".svg-progress{position:fixed;top:0;left:0;height:100%;width:100%;box-sizing:border-box;transform:scaleY(1.2)}#progress-bar{transition:stroke .3s linear}.progress-complete{opacity:0;transition:opacity .2s linear}.waiting{position:fixed;width:100%;height:100%;background-color:#000;top:0;left:0;display:flex;justify-content:center;align-items:center;color:#fff;z-index:2}"]
    })
], ProgressComponent);

let NgNaatLivenessModule = class NgNaatLivenessModule {
};
NgNaatLivenessModule = __decorate([
    NgModule({
        declarations: [NgNaatLivenessComponent, ProgressComponent],
        imports: [
            CommonModule
        ],
        exports: [NgNaatLivenessComponent]
    })
], NgNaatLivenessModule);

/*
 * Public API Surface of ng-naat-liveness
 */

/**
 * Generated bundle index. Do not edit.
 */

export { NgNaatLivenessComponent, NgNaatLivenessModule, LandmarksService as ɵa, ProgressComponent as ɵb };
//# sourceMappingURL=ng-naat-liveness.js.map
